﻿package wck {
	
	import Box2DAS.*;
	import Box2DAS.Collision.*;
	import Box2DAS.Collision.Shapes.*;
	import Box2DAS.Common.*;
	import Box2DAS.Dynamics.*;
	import Box2DAS.Dynamics.Contacts.*;
	import Box2DAS.Dynamics.Joints.*;
	import cmodule.Box2D.*;
	import flash.events.*;
	
	/**
	 * This class can be used to filter contact events based on certain contact properties. Pass a contact to 
	 * the filter() method to see if it matches the specified conditions.
	 */
	public class ContactEventFilter {
		
		/// Exclude if the other fixture is a sensor?
		public var excludeSensors:Boolean = true;
		
		/// Exclude if the contact is not touching?
		public var excludeNotTouching:Boolean = true;
		
		/// Exclude if the contact has been set as a sensor? This is a method of disabling
		/// ths contact and can happen between two non sensor fixtures.
		/// public var excludeSetSensor:Boolean = true;
		
		/// Exclude if the contact has been disabled.
		public var excludeDisabled:Boolean = true;
		
		/// Exclude if the other fixture / body has been destroyed.
		public var excludeDestroyed:Boolean = true;
		
		public var categoryBits:int = 0x0001;
		public var maskBits:int = 0xFFFF;
		public var groupIndex:int = 0;
		
		/**
		 * Does the contact event match the conditions we're looking for?
		 */
		public function filter(e:ContactEvent):Boolean {
			if(excludeNotTouching && !e.contact.IsTouching()) {
				return false;
			}
			if(excludeDisabled && !e.contact.IsEnabled()) {
				return false;
			}
			return filterFixture(e.other);
		}
		
		public function filterFixture(f:b2Fixture):Boolean {
			if(excludeDestroyed && !(f._ptr && f.m_body._ptr)) {
				return false;
			}			
			if(excludeSensors && f.IsSensor()) {
				return false;
			}
			var flt:b2Filter = f.m_filter;
			if(groupIndex == flt.groupIndex && groupIndex != 0) {
				return groupIndex > 0;
			}
			return (maskBits & flt.categoryBits) != 0 && (categoryBits & flt.maskBits) != 0;
		}
	}
}